@@ -10,7 +10,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import WebKit2, Gtk, Gio, Gdk, GLib
+from gi.repository import WebKit2, Gtk, Gio, GLib
 
 from urllib.parse import urlparse
 from time import time
@@ -91,19 +91,19 @@ class WebView(WebKit2.WebView):
             Update zoom level
         """
         try:
-            zoom_level = App().websettings.get_zoom(self.uri)
-            if zoom_level is None:
-                zoom_level = 100
-            if self.__related_view is None:
-                zoom_level *= self._window.zoom_level
-            else:
+            zoom_level = self._window.zoom_level
+            if self.__related_view is not None:
                 window = self.__related_view.get_ancestor(Gtk.Window)
                 if window is not None and hasattr(window, "zoom_level"):
-                    zoom_level *= window.zoom_level
+                    zoom_level = window.zoom_level
+            else:
+                _zoom_level = App().websettings.get_zoom(self.uri)
+                if _zoom_level is not None:
+                    zoom_level = _zoom_level / 100
         except Exception as e:
             Logger.error("WebView::update_zoom_level(): %s", e)
         Logger.debug("Update zoom level: %s", zoom_level)
-        self.set_zoom_level(zoom_level / 100)
+        self.set_zoom_level(zoom_level)
 
     def print(self):
         """
@@ -119,7 +119,7 @@ class WebView(WebKit2.WebView):
         """
         current = App().websettings.get_zoom(self.uri)
         if current is None:
-            current = 100
+            current = int(self._window.zoom_level * 100)
         current += 10
         App().websettings.set_zoom(current, self.uri)
         self.update_zoom_level()
@@ -132,7 +132,7 @@ class WebView(WebKit2.WebView):
         """
         current = App().websettings.get_zoom(self.uri)
         if current is None:
-            current = 100
+            current = int(self._window.zoom_level * 100)
         current -= 10
         if current == 0:
             return 10
@@ -145,8 +145,10 @@ class WebView(WebKit2.WebView):
             Reset zoom level
             @return current zoom after zoom out
         """
-        App().websettings.set_zoom(100, self.uri)
+        current = int(self._window.zoom_level * 100)
+        App().websettings.set_zoom(None, self.uri)
         self.update_zoom_level()
+        return current
 
     def set_title(self, title):
         """
@@ -326,6 +328,14 @@ class WebView(WebKit2.WebView):
         """
         return self.__content_manager
 
+    @property
+    def readable(self):
+        """
+            True if webview readable
+            @return bool
+        """
+        return self._readable
+
     @property
     def context(self):
         """
@@ -423,14 +433,6 @@ class WebView(WebKit2.WebView):
         """
         return self.__selection
 
-    @property
-    def readable_content(self):
-        """
-            Readable content
-            @return content as str
-        """
-        return self._readable_content
-
 #######################
 # PRIVATE             #
 #######################
@@ -444,7 +446,7 @@ class WebView(WebKit2.WebView):
             @param view as View
         """
         WebViewErrors.__init__(self)
-        WebViewNavigation.__init__(self, related_view)
+        WebViewNavigation.__init__(self)
         WebViewSignals.__init__(self)
         self.__context = context
         WebViewArtwork.__init__(self)
@@ -458,67 +460,77 @@ class WebView(WebKit2.WebView):
         # WebKitGTK doesn't provide an API to get selection, so try to guess
         # it from clipboard FIXME Get it from extensions
         self.__selection = ""
-        self._readable_content = ""
+        self._readable = False
         self.__uri = None
+        self._initial_uri = None
         self._title = None
         self.__related_view = related_view
         self._shown = False
         self.set_hexpand(True)
         self.set_vexpand(True)
         self.clear_text_entry()
-        # Set settings
-        settings = self.get_settings()
-        system = Gio.Settings.new("org.gnome.desktop.interface")
-        animations = system.get_value("enable-animations")
-        settings.set_property("enable-java",
-                              App().settings.get_value('enable-plugins'))
-        settings.set_property("enable-plugins",
-                              App().settings.get_value('enable-plugins'))
-        settings.set_property("minimum-font-size",
-                              App().settings.get_value(
-                                  "min-font-size").get_int32())
-        if App().settings.get_value("use-system-fonts"):
-            self.__set_system_fonts(settings, system)
-        else:
-            settings.set_property("monospace-font-family",
+        if related_view is None:
+            # Set settings
+            settings = self.get_settings()
+            settings.set_property("enable-java",
+                                  App().settings.get_value('enable-plugins'))
+            settings.set_property("enable-plugins",
+                                  App().settings.get_value('enable-plugins'))
+            settings.set_property("minimum-font-size",
                                   App().settings.get_value(
-                                      "font-monospace").get_string())
-            settings.set_property("sans-serif-font-family",
-                                  App().settings.get_value(
-                                      "font-sans-serif").get_string())
-            settings.set_property("serif-font-family",
-                                  App().settings.get_value(
-                                      "font-serif").get_string())
-        settings.set_property("auto-load-images", True)
-        settings.set_property("enable-site-specific-quirks", True)
-        settings.set_property("allow-universal-access-from-file-urls", False)
-        settings.set_property("allow-file-access-from-file-urls", False)
-        settings.set_property("enable-javascript", True)
-        settings.set_property("enable-media-stream", True)
-        settings.set_property("enable-mediasource", False)
-        settings.set_property("enable-developer-extras",
-                              App().settings.get_value("developer-extras"))
-        settings.set_property("enable-offline-web-application-cache", True)
-        settings.set_property("enable-page-cache", True)
-        settings.set_property("enable-resizable-text-areas", True)
-        settings.set_property("enable-smooth-scrolling", animations)
-        settings.set_property("enable-webaudio", True)
-        settings.set_property("enable-webgl", True)
-        settings.set_property("javascript-can-access-clipboard", True)
-        settings.set_property("javascript-can-open-windows-automatically",
-                              True)
-        settings.set_property("media-playback-allows-inline", True)
+                                      "min-font-size").get_int32())
+            if App().settings.get_value("use-system-fonts"):
+                self.__set_system_fonts(settings)
+            else:
+                settings.set_property("monospace-font-family",
+                                      App().settings.get_value(
+                                          "font-monospace").get_string())
+                settings.set_property("sans-serif-font-family",
+                                      App().settings.get_value(
+                                          "font-sans-serif").get_string())
+                settings.set_property("serif-font-family",
+                                      App().settings.get_value(
+                                          "font-serif").get_string())
+            settings.set_property("auto-load-images", True)
+            settings.set_property("enable-site-specific-quirks", True)
+            settings.set_property("allow-universal-access-from-file-urls",
+                                  False)
+            settings.set_property("allow-file-access-from-file-urls", False)
+            settings.set_property("enable-javascript", True)
+            settings.set_property("enable-media-stream", True)
+            settings.set_property("enable-mediasource", False)
+            autoplay_videos = App().settings.get_value('autoplay-videos')
+            # Setting option to False make video playback buggy
+            if not autoplay_videos:
+                settings.set_property("media-playback-requires-user-gesture",
+                                      True)
+            settings.set_property("enable-developer-extras",
+                                  App().settings.get_value("developer-extras"))
+            settings.set_property("enable-offline-web-application-cache", True)
+            settings.set_property("enable-page-cache", True)
+            settings.set_property("enable-resizable-text-areas", True)
+            settings.set_property(
+                "enable-smooth-scrolling",
+                App().settings.get_value("enable-smooth-scrolling"))
+            settings.set_property("enable-webaudio", True)
+            settings.set_property("enable-webgl", True)
+            settings.set_property("javascript-can-access-clipboard", True)
+            settings.set_property("javascript-can-open-windows-automatically",
+                                  True)
+            settings.set_property("media-playback-allows-inline", True)
+            if WebKit2.get_minor_version() > 22:
+                settings.set_property(
+                    "enable-back-forward-navigation-gestures", True)
         self.connect("create", self.__on_create)
         self.connect("load-changed", self._on_load_changed)
 
-    def __set_system_fonts(self, settings, system=None):
+    def __set_system_fonts(self, settings):
         """
             Set system font
             @param settings as WebKit2.Settings
             @param system as Gio.Settings("org.gnome.desktop.interface")
         """
-        if system is None:
-            system = Gio.Settings.new("org.gnome.desktop.interface")
+        system = Gio.Settings.new("org.gnome.desktop.interface")
         settings.set_property(
             "monospace-font-family",
             system.get_value("monospace-font-name").get_string())
@@ -555,54 +567,28 @@ class WebView(WebKit2.WebView):
             @param navigation_action as WebKit2.NavigationAction
             @param elapsed as float
         """
-        def on_load_changed(webview, event):
-            parsed = urlparse(webview.uri)
-            # First, adblocking
-            if not parsed.netloc:
-                return
-            if App().settings.get_value("adblock") and\
-                    parsed.scheme in ["http", "https"] and\
-                    not App().adblock_exceptions.find_parsed(parsed):
-                if App().adblock.is_netloc_blocked(parsed.netloc) or\
-                        App().adblock.is_uri_blocked(webview.uri,
-                                                     parsed.netloc):
-                    webview.destroy()
-                    return
-            if event != WebKit2.LoadEvent.FINISHED:
-                return
-            else:
-                webview.disconnect_by_func(on_load_changed)
-
-            # Then popup blocking
-            popup_block = App().settings.get_value("popupblock")
-            if related.uri is not None:
-                parsed_related = urlparse(related.uri)
-                exception = elapsed < 1.0 or\
-                    App().popup_exceptions.find_parsed(parsed_related)
-            else:
-                exception = False
-
-            properties = webview.get_window_properties()
-            if not exception and popup_block and\
-                    navigation_action.get_navigation_type() in [
-                        WebKit2.NavigationType.OTHER,
-                        WebKit2.NavigationType.RELOAD,
-                        WebKit2.NavigationType.BACK_FORWARD]:
-                related.add_popup(webview)
-                webview.connect("close", self.__on_popup_close, related)
-                if related == self._window.container.current.webview:
-                    self._window.toolbar.title.show_indicator(
-                        Indicator.POPUPS)
-            elif (properties.get_toolbar_visible() or
-                    properties.get_scrollbars_visible()) and\
-                    not self._window.modifiers == Gdk.KEY_Shift_L:
-                self._window.container.add_webview_with_new_view(
-                    webview,
-                    LoadingType.FOREGROUND)
-            else:
-                self._window.container.popup_webview(webview, True)
-
-        webview.connect("load-changed", on_load_changed)
+        popup_block = App().settings.get_value("popupblock")
+        parsed = urlparse(webview.uri)
+        exception = elapsed < 1.0 or\
+            App().popup_exceptions.find_parsed(parsed)
+        properties = webview.get_window_properties()
+        if not exception and popup_block and\
+                navigation_action.get_navigation_type() in [
+                    WebKit2.NavigationType.OTHER,
+                    WebKit2.NavigationType.RELOAD,
+                    WebKit2.NavigationType.BACK_FORWARD]:
+            related.add_popup(webview)
+            webview.connect("close", self.__on_popup_close, related)
+            if related == self._window.container.current.webview:
+                self._window.toolbar.title.show_indicator(
+                    Indicator.POPUPS)
+        elif (properties.get_toolbar_visible() or
+                properties.get_scrollbars_visible()):
+            self._window.container.add_webview_with_new_view(
+                webview,
+                LoadingType.FOREGROUND)
+        else:
+            self._window.container.popup_webview(webview, True)
 
     def __on_popup_close(self, webview, related):
         """
